	list p=16f628
;**************************************************************
;*
;*	----------------------------------	
;*	PORTA: 	0 -
;*		1 RC-5 Output
;*		2 -	
;*		3 -
;*		4 -
;*
;*	PORTB:	0 < 0: 0/9  1: leise/laut
;*		1 > 
;*		2 > 
;*		3 > 
;*		4 < Taste 4
;*		5 < Taste 5
;*		6 < Taste 6
;*		7 < Taste 7
;*
;**************************************************************
;
; Senden von RC-5-codes an philips/marantz gerte
;
; An RB4..7 liegen 4 Tasten nach Masse
; beim Drcken einer Taste wird ein der Taste
; fest zugeordneter RC-5-code gesendet
;
; output: RA1
;
; Takt 4 MHz
;
;
; RC5-code hatt 5 Adressenbits (A0-A4) und 7 Commandobits (C0-C6)
; und ist wie folgt als 14-bit code aufgebaut:
;  1. startbit=1
;  2. invertiertes C6
;  3. Toggelbit: wechselt seinen wert von code zu code
;  4. A4
;  5. A3
;  6. A2
;  7. A1
;  8. A0
;  9. C5
; 10. C4
; 11. C3
; 12. C2
; 13. C1
; 14. C0
;
;
;
;****************************************************************************

	#include <P16f628.INC>

	ERRORLEVEL      -302    	;SUPPRESS BANK SELECTION MESSAGES

; Configuration festlegen:
; Power on Timer, kein Watchdog, XT-Oscillator, kein Brown out, kein LV-programming
;	__CONFIG	_PWRTE_ON & _WDT_OFF & _XT_OSC & _BODEN_OFF & _LVP_OFF
; Power on Timer, kein Watchdog, interner 4MHz-Oscillator, kein Brown out, kein LV-programming
	__CONFIG	_MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _BODEN_OFF & _LVP_OFF


toggel	Equ	0x20		; zhlt ir-Sendungen, bit 0 ist toggelbit
gr	Equ	0x21		; Gruppencode  5 bit rechtsbndig
co	Equ	0x22		; Commandocode 6 bit rechtsbndig
cycl_1	Equ	0x23		; zhler 
cycl_2	Equ	0x24		; zhler

#define	IrOut	PORTA, 1
#define Auswahl	PORTB, 0

;****************************************************************************
	org 	0

Init
; ports einstellen
	bsf     STATUS, RP0	; Bank 1
	movlw	B'00111101'	; RA1 output 
	movwf	TRISA		; 
	movlw	B'11110000'	; PortB7..4 input, 3..0 output
	movwf	TRISB
	bcf     STATUS, RP0	; Bank 0
	clrf	PORTA		; 
	clrf	PORTB		;

; 16F628 alle Comparatoreingnge auf Digital umschalten
	BSF	CMCON, CM0
	BSF	CMCON, CM1
	BSF	CMCON, CM2

; interrupt on change an PortB RB4..7
	bsf     STATUS, RP0	; Bank 1
	bcf	OPTION_REG, 7	; pull up an
	bcf     STATUS, RP0	; Bank 0
	clrf	INTCON		; GIE aus
	bsf	INTCON, RBIE	; RBIF kann wecken

; pull-up ein
	bcf	INTCON, RBIF

Main
	sleep
	; wecken durch Tastendruck an PortB
	btfss	INTCON, RBIF
	goto	Main		; str mich nicht!
	movfw	PORTB
	bcf	INTCON, RBIF

	btfss	PORTB,4
	call	Taste4
	btfss	PORTB,5
	call	Taste5
	btfss	PORTB,6
	call	Taste6
	btfss	PORTB,7
	call	Taste7
	goto	Main

;****************************************************************************
Taste4
	movlw	D'0'		; TV
	movwf	gr
	movlw	D'32'		; Kanal +
	movwf	co
	call	Tx
	return
Taste5
	movlw	D'0'		; TV
	movwf	gr
	movlw	D'33'		; Kanal -
	movwf	co
	call	Tx
	return
Taste6
	movlw	D'0'		; TV
	movwf	gr
	movlw	D'16'		; lauter
	btfss	Auswahl
	movlw	D'9'		; 9
	movwf	co
	call	Tx
	return
Taste7
	movlw	D'0'		; TV
	movwf	gr
	movlw	D'17'		; leiser
	btfss	Auswahl
	movlw	0		; 0
	movwf	co
	call	Tx
	return


;****************************************************************************
; Senden eines kompletten IR-Befehls mit Gruppencode in gr und Kommando in co
; die Routine ist etwas primitiv aufgebaut, dafr aber bersichtlich
Tx
	incf	toggel, f

	;Startbit
	call	TxEins

	;invertiertes C6
	btfss	co,6		;1T
	call	TxEins		;2T+
	btfsc	co,6		;2T
	call	TxNull		;0T

	;Toggelbit
	btfsc	toggel,0
	call	TxEins
	btfss	toggel,0
	call	TxNull

	;A4,3,2,1,0
	btfsc	gr,4
	call	TxEins
	btfss	gr,4
	call	TxNull

	btfsc	gr,3
	call	TxEins
	btfss	gr,3
	call	TxNull

	btfsc	gr,2
	call	TxEins
	btfss	gr,2
	call	TxNull

	btfsc	gr,1
	call	TxEins
	btfss	gr,1
	call	TxNull

	btfsc	gr,0
	call	TxEins
	btfss	gr,0
	call	TxNull

	;C5,4,3,2,1,0
	
	btfsc	co,5
	call	TxEins
	btfss	co,5
	call	TxNull

	btfsc	co,4
	call	TxEins
	btfss	co,4
	call	TxNull

	btfsc	co,3
	call	TxEins
	btfss	co,3
	call	TxNull

	btfsc	co,2
	call	TxEins
	btfss	co,2
	call	TxNull

	btfsc	co,1
	call	TxEins
	btfss	co,1
	call	TxNull

	btfsc	co,0
	call	TxEins
	btfss	co,0
	call	TxNull

	return

;****************************************************************************
; Senden einer 0 (on-off)
TxNull
	call	LichtAn		;2T
	call	LichtAus	;2T
	return			;2T

;****************************************************************************
; Senden einer 1 (off-on)	
TxEins
	call	LichtAus
	call	LichtAn
	return

;****************************************************************************
; Licht aus  (889 us)
; ca. 13 Takte werden fr den Aufruf bentigt
;  889 us -13 us =876 us = 175 Takte x 5
LichtAus

	movlw	D'175'
	movwf	cycl_2
LichtAus2
	nop			;1T
	nop			;1T
	decfsz	cycl_2,f	;1T
	goto	LichtAus2	;2T
	return			;2T

;****************************************************************************
; Licht pulsen (889 us)
; ideal: 32 IR Pulse von je 6,944 us mit Pausen von 20,8332 us
; real:  32 Pulse a 7 us mit je 21 us Pause, letzte Pause entfllt (32x7+31*21=875 us)
LichtAn
	; 1. Puls
	bsf	IrOut
	movlw	D'31'		; 31 weitere Zyklen
	movwf	cycl_1
	nop
	nop
	nop
	nop
	bcf	IrOut

	;31 Zyklen aus Pause und Puls
LichtAn1
	;Pause
	movlw	D'5'		;1T
	movwf	cycl_2		;1T
	nop			;1T
	nop			;1T
LichtAn2			; 4x3T + 1x2T =14T
	decfsz	cycl_2,f	;1T
	goto	LichtAn2	;2T
	;Puls			;7T
	bsf	IrOut
	nop
	nop
	nop
	nop
	nop
	nop
	bcf	IrOut
	decfsz	cycl_1,f	;1T
	goto	LichtAn1	;2T

	return

	end





